@@ -35,7 +35,8 @@
 import numpy as np
 import sys
 from phonopy.structure.cells import (get_reduced_bases,
-                                     get_equivalent_smallest_vectors)
+                                     get_equivalent_smallest_vectors,
+                                     compute_permutation_for_rotation)
 
 def get_force_constants(set_of_forces,
                         symmetry,
@@ -86,31 +87,23 @@ def get_fc2(supercell,
         symmetry,
         computation_algorithm=computation_algorithm)
 
-    # Distribute non-equivalent force constants to those equivalent
-    symprec = symmetry.get_symmetry_tolerance()
     rotations = symmetry.get_symmetry_operations()['rotations']
-    trans = symmetry.get_symmetry_operations()['translations']
-    positions = supercell.get_scaled_positions()
     lattice = np.array(supercell.get_cell().T, dtype='double', order='C')
-
+    permutations = symmetry.get_atomic_permutations()
     if atom_list is None:
         distribute_force_constants(force_constants,
                                    range(supercell.get_number_of_atoms()),
                                    atom_list_done,
                                    lattice,
-                                   positions,
                                    rotations,
-                                   trans,
-                                   symprec)
+                                   permutations)
     else:
         distribute_force_constants(force_constants,
                                    atom_list,
                                    atom_list_done,
                                    lattice,
-                                   positions,
                                    rotations,
-                                   trans,
-                                   symprec)
+                                   permutations)
 
     if decimals:
         force_constants = force_constants.round(decimals=decimals)
@@ -163,9 +156,7 @@ def symmetrize_force_constants(force_constants, iteration=1):
 
 def symmetrize_compact_force_constants(force_constants,
                                        supercell,
-                                       symmetry,
-                                       s2p_map,
-                                       p2s_map,
+                                       primitive,
                                        level=2):
     """Symmetry force constants by translational and permutation symmetries.
 
@@ -181,29 +172,21 @@ def symmetrize_compact_force_constants(force_constants,
 
     """
 
-    rotations = []
-    trans = []
-    identity = np.eye(3, dtype='intc')
-
-    for r, t in zip(symmetry.get_symmetry_operations()['rotations'],
-                    symmetry.get_symmetry_operations()['translations']):
-        if (r == identity).all():
-            rotations.append(r)
-            trans.append(t)
-
-    permutations = _compute_all_sg_permutations(
-        supercell.get_scaled_positions(),
-        np.array(rotations, dtype='intc', order='C'),
-        np.array(trans, dtype='double'),
-        np.array(supercell.get_cell().T, dtype='double', order='C'),
-        symmetry.get_symmetry_tolerance())
+    s2p_map = primitive.get_supercell_to_primitive_map()
+    p2s_map = primitive.get_primitive_to_supercell_map()
+    p2p_map = primitive.get_primitive_to_primitive_map()
+    permutations = primitive.get_atomic_permutations()
+    s2pp_map, nsym_list = get_nsym_list_and_s2pp(s2p_map,
+                                                 p2p_map,
+                                                 permutations)
 
     try:
         import phonopy._phonopy as phonoc
         phonoc.perm_trans_symmetrize_compact_fc(force_constants,
                                                 permutations,
-                                                s2p_map,
+                                                s2pp_map,
                                                 p2s_map,
+                                                nsym_list,
                                                 level)
     except ImportError:
         print("Import error at phonoc.perm_trans_symmetrize_compact_fc.")
@@ -215,19 +198,10 @@ def distribute_force_constants(force_constants,
                                atom_list,
                                atom_list_done,
                                lattice, # column vectors
-                               positions, # scaled (fractional)
                                rotations, # scaled (fractional)
-                               trans, # scaled (fractional)
-                               symprec):
-    permutations = _compute_all_sg_permutations(positions,
-                                                rotations,
-                                                trans,
-                                                lattice,
-                                                symprec)
-
+                               permutations):
     map_atoms, map_syms = _get_sym_mappings_from_permutations(
         permutations, atom_list_done)
-
     rots_cartesian = np.array([similarity_transformation(lattice, r)
                                for r in rotations],
                               dtype='double', order='C')
@@ -276,10 +250,10 @@ def get_positions_sent_by_rot_inv(lattice, # column vectors
     for sym in site_symmetry:
         # inverse permutation of sym;
         # satisfies 'rotated_positions[rot_map] == positions'
-        rot_map = _compute_permutation_for_rotation(np.dot(positions, sym.T),
-                                                    positions,
-                                                    lattice,
-                                                    symprec)
+        rot_map = compute_permutation_for_rotation(np.dot(positions, sym.T),
+                                                   positions,
+                                                   lattice,
+                                                   symprec)
         rot_map_syms.append(rot_map)
 
     return np.array(rot_map_syms, dtype='intc', order='C')
@@ -386,14 +360,13 @@ def set_tensor_symmetry(force_constants,
 
         assert num_equiv_atoms * num_sitesym == len(rotations)
 
+    permutations = symmetry.get_atomic_permutations()
     distribute_force_constants(fc_new,
                                range(len(positions)),
                                indep_atoms,
                                lattice,
-                               positions,
                                rotations,
-                               translations,
-                               symprec)
+                               permutations)
 
     force_constants[:] = fc_new
 
@@ -546,22 +519,55 @@ def similarity_transformation(rot, mat):
     return np.dot(rot, np.dot(mat, np.linalg.inv(rot)))
 
 def show_drift_force_constants(force_constants,
+                               supercell=None,
+                               primitive=None,
                                name="force constants",
                                values_only=False):
-    num_atom = force_constants.shape[0]
-    maxval1 = 0
-    maxval2 = 0
-    jk1 = [0, 0]
-    jk2 = [0, 0]
-    for i, j, k in list(np.ndindex((num_atom, 3, 3))):
-        val1 = force_constants[:, i, j, k].sum()
-        val2 = force_constants[i, :, j, k].sum()
-        if abs(val1) > abs(maxval1):
-            maxval1 = val1
-            jk1 = [j, k]
-        if abs(val2) > abs(maxval2):
-            maxval2 = val2
-            jk2 = [j, k]
+    if force_constants.shape[0] == force_constants.shape[1]:
+        num_atom = force_constants.shape[0]
+        maxval1 = 0
+        maxval2 = 0
+        jk1 = [0, 0]
+        jk2 = [0, 0]
+        for i, j, k in list(np.ndindex((num_atom, 3, 3))):
+            val1 = force_constants[:, i, j, k].sum()
+            val2 = force_constants[i, :, j, k].sum()
+            if abs(val1) > abs(maxval1):
+                maxval1 = val1
+                jk1 = [j, k]
+            if abs(val2) > abs(maxval2):
+                maxval2 = val2
+                jk2 = [j, k]
+    else:
+        s2p_map = primitive.get_supercell_to_primitive_map()
+        p2s_map = primitive.get_primitive_to_supercell_map()
+        p2p_map = primitive.get_primitive_to_primitive_map()
+        permutations = primitive.get_atomic_permutations()
+        s2pp_map, nsym_list = get_nsym_list_and_s2pp(s2p_map,
+                                                     p2p_map,
+                                                     permutations)
+
+        try:
+            import phonopy._phonopy as phonoc
+            phonoc.transpose_compact_fc(force_constants,
+                                        permutations,
+                                        s2pp_map,
+                                        p2s_map,
+                                        nsym_list)
+            maxval1, jk1 = _get_drift_per_index(force_constants)
+            phonoc.transpose_compact_fc(force_constants,
+                                        permutations,
+                                        s2pp_map,
+                                        p2s_map,
+                                        nsym_list)
+            maxval2, jk2 = _get_drift_per_index(force_constants)
+
+        except ImportError:
+            print("Import error at phonoc.tranpose_compact_fc.")
+            print("Corresponding pytono code is not implemented.")
+            import sys
+            sys.exit(1)
+
     if values_only:
         text = ""
     else:
@@ -571,6 +577,25 @@ def show_drift_force_constants(force_constants,
     print(text)
 
 
+def get_nsym_list_and_s2pp(s2p_map,
+                           p2p_map,
+                           permutations):
+    s2pp = np.array([p2p_map[i] for i in s2p_map], dtype='intc')
+    nsym_list = np.array([np.where(permutations[:, i] == target)[0][0]
+                          for i, target in enumerate(s2p_map)], dtype='intc')
+    return s2pp, nsym_list
+
+def _get_drift_per_index(force_constants):
+    num_atom = force_constants.shape[0]
+    maxval = 0
+    jk = [0, 0]
+    for i, j, k in list(np.ndindex((num_atom, 3, 3))):
+        val = force_constants[i, :, j, k].sum()
+        if abs(val) > abs(maxval):
+            maxval = val
+            jk = [j, k]
+    return maxval, jk
+
 #################
 # Local methods #
 #################
@@ -804,120 +829,6 @@ def _get_shortest_distance_in_PBC(pos_i, pos_j, reduced_bases):
                 distances.append(np.linalg.norm(np.dot(diff, reduced_bases)))
     return np.min(distances)
 
-def _compute_all_sg_permutations(positions, # scaled positions
-                                 rotations, # scaled
-                                 translations, # scaled
-                                 lattice, # column vectors
-                                 symprec):
-    """Compute a permutation for every space group operation.
-
-    See '_compute_permutation_for_rotation' for more info.
-
-    Output has shape (num_rot, num_pos)
-
-    """
-
-    out = [] # Finally the shape is fixed as (num_sym, num_pos_of_supercell).
-    for (sym, t) in zip(rotations, translations):
-        rotated_positions = np.dot(positions, sym.T) + t
-        out.append(_compute_permutation_for_rotation(positions,
-                                                     rotated_positions,
-                                                     lattice,
-                                                     symprec))
-    return np.array(out, dtype='intc', order='C')
-
-def _compute_permutation_for_rotation(positions_a, # scaled positions
-                                      positions_b,
-                                      lattice, # column vectors
-                                      symprec):
-    """Get the overall permutation such that
-
-        positions_a[perm[i]] == positions_b[i]   (modulo the lattice)
-
-    or in numpy speak,
-
-        positions_a[perm] == positions_b   (modulo the lattice)
-
-    This version is optimized for the case where positions_a and positions_b
-    are related by a rotation.
-
-    """
-
-    # Sort both sides by some measure which is likely to produce a small
-    # maximum value of (sorted_rotated_index - sorted_original_index).
-    # The C code is optimized for this case, reducing an O(n^2)
-    # search down to ~O(n). (for O(n log n) work overall, including the sort)
-    #
-    # We choose distance from the nearest bravais lattice point as our measure.
-    def sort_by_lattice_distance(fracs):
-        carts = np.dot(fracs - np.rint(fracs), lattice.T)
-        perm = np.argsort(np.sum(carts**2, axis=1))
-        sorted_fracs = np.array(fracs[perm], dtype='double', order='C')
-        return perm, sorted_fracs
-
-    (perm_a, sorted_a) = sort_by_lattice_distance(positions_a)
-    (perm_b, sorted_b) = sort_by_lattice_distance(positions_b)
-
-    # Call the C code on our conditioned inputs.
-    perm_between = _compute_permutation_c(sorted_a,
-                                          sorted_b,
-                                          lattice,
-                                          symprec)
-
-    # Compose all of the permutations for the full permutation.
-    #
-    # Note the following properties of permutation arrays:
-    #
-    # 1. Inverse:         if  x[perm] == y  then  x == y[argsort(perm)]
-    # 2. Associativity:   x[p][q] == x[p[q]]
-    return perm_a[perm_between][np.argsort(perm_b)]
-
-def _compute_permutation_c(positions_a, # scaled positions
-                           positions_b,
-                           lattice, # column vectors
-                           symprec):
-    """Version of '_compute_permutation_for_rotation' which just directly
-    calls the C function, without any conditioning of the data.
-    Skipping the conditioning step makes this EXTREMELY slow on large
-    structures.
-
-    """
-
-    permutation = np.zeros(shape=(len(positions_a),), dtype='intc')
-
-    def permutation_error():
-        raise ValueError("Input forces are not enough to calculate force constants, "
-                         "or something wrong (e.g. crystal structure does not match).")
-
-    try:
-        import phonopy._phonopy as phonoc
-        is_found = phonoc.compute_permutation(permutation,
-                                              lattice,
-                                              positions_a,
-                                              positions_b,
-                                              symprec)
-
-        if not is_found:
-            permutation_error()
-
-    except ImportError:
-        for i, pos_b in enumerate(positions_b):
-            diffs = positions_a - pos_b
-            diffs -= np.rint(diffs)
-            diffs = np.dot(diffs, lattice.T)
-
-            possible_j = np.nonzero(
-                np.sqrt(np.sum(diffs**2, axis=1)) < symprec)[0]
-            if len(possible_j) != 1:
-                permutation_error()
-
-            permutation[i] = possible_j[0]
-
-        if -1 in permutation:
-            permutation_error()
-
-    return permutation
-
 def _get_sym_mappings_from_permutations(permutations,
                                         atom_list_done):
 
